import React, {useContext, useState} from "react";
import {ArrowsClockwise, Bug, BugDroid} from "@phosphor-icons/react";
import {useRouteError} from "react-router-dom";

/**
 * Application error context.
 */
const ApplicationErrorContext = React.createContext<Error>(undefined);

/**
 * Get current error from context or router.
 */
export function useApplicationError(): Error
{
	// Get error from context or router.
	const error = useContext(ApplicationErrorContext);
	const routeError = useRouteError() as Error;
	return error ?? routeError;
}

/**
 * Application error component.
 */
export function ApplicationError({children}: {
	children?: (error: Error) => React.ReactElement;
})
{
	// Get error from context.
	const error = useApplicationError();

	// Show details state.
	const [showDetails, setShowDetails] = useState(false);

	return (
		<main className={"error"}>
			{children ? children(error) : (
				<>
					<Bug size={64} weight={"duotone"} />
					<h1>Error</h1>

					<hr />

					<h2>{error.name}</h2>

					<p>An unexpected error happened and the application was forced to quit.</p>

					<pre className={"error"}>{error.message}</pre>

					<button onClick={() => window.location.reload()}><ArrowsClockwise size={20} /> Restart application</button>

					<div className={"details"}>
						<button onClick={() => setShowDetails(!showDetails)}><BugDroid size={18} /> {showDetails ? "Hide" : "Show"} details</button>

						{ // Show details if required.
							showDetails && (
								<pre>
									{error.stack}
								</pre>
							)
						}
					</div>
				</>
			)}
		</main>
	);
}

/**
 * Error boundary component for the application.
 */
export class ApplicationErrorBoundary extends React.Component<React.PropsWithChildren<{
	errorElement: React.ReactNode;
}>, {
	error: Error;
}>
{
	static getDerivedStateFromError(error: Error)
	{
		return { error: error };
	}

	render()
	{
		if (this.state?.error)
			// An error happened, showing the application error content.
			return (
				<ApplicationErrorContext.Provider value={this.state?.error}>
					{this.props.errorElement}
				</ApplicationErrorContext.Provider>
			);

		return this.props.children;
	}
}
