import React, {useEffect, useState} from "react";
import {CaretLeft, CaretRight} from "@phosphor-icons/react";
import {Tooltip} from "../Floating/Tooltip";
import {usePreviousValue} from "../../Utils";

/**
 * Pagination component.
 */
export function Pagination({ page, onChange, count }: {
	/**
	 * The current page.
	 */
	page: number;

	/**
	 * Called when a new page is selected.
	 * @param newPage The newly selected page.
	 */
	onChange: (newPage: number) => void;

	/**
	 * Pages count.
	 */
	count: number;
})
{
	// Memorize previous page.
	const previousPage = usePreviousValue(page);

	// The input text to use, when the currently entered value is not a number.
	const [inputText, setInputText] = useState(undefined);

	useEffect(() => {
		if (page != previousPage)
			// If the page has changed, resetting the input text.
			setInputText(undefined);
	}, [previousPage, page]);

	return (
		<nav className={"pages"}>
			<Tooltip content={"Previous"}>
				<button className={"flat icon-only"} disabled={page <= 1} onClick={() => onChange(page - 1)}>
					<CaretLeft/>
				</button>
			</Tooltip>

			<ul>
				{ // First page, only show when the current page isn't the first page.
					page > 1 &&
					<li>
						<button className={"flat"} onClick={() => onChange(1)}>1</button>
					</li>
				}

				{ // Space between first page and shown pages.
					// Only show when the page isn't the first page.
					page - 4 > 1 &&
					<li>
						<button className={"flat"} onClick={page - 4 > 2 ? undefined : () => onChange(page - 4)}>{page - 4 > 2 ? "···" : page - 4}</button>
					</li>
				}

				{ // Only show when the page isn't the first page.
					page - 3 > 1 &&
					<li>
						<button className={"flat"} onClick={() => onChange(page - 3)}>{page - 3}</button>
					</li>
				}
				{ // Only show when the page isn't the first page.
					page - 2 > 1 &&
					<li>
						<button className={"flat"} onClick={() => onChange(page - 2)}>{page - 2}</button>
					</li>
				}
				{ // Only show when the page isn't the first page.
					page - 1 > 1 &&
					<li>
						<button className={"flat"} onClick={() => onChange(page - 1)}>{page - 1}</button>
					</li>
				}

				{ // Current page input.
					<li className={"current"}>
						<input type={"number"} step={1} min={1} max={count} value={inputText ?? page} onChange={(event) => {
							// Try to get the new value, between 1 and page count.
							const newValue = Math.max(Math.min(event.currentTarget.valueAsNumber, count), 1);
							if (isNaN(newValue))
								// Not a numeric value, keep the current input text without changing the page number.
								setInputText(event.currentTarget.value);
							else
							{ // The value has been read successfully, changing it and resetting input text.
								onChange(newValue);
								setInputText(undefined);
							}
						}} />
					</li>
				}

				{ // Only show when the page isn't the last page.
					page + 1 < count &&
					<li>
						<button className={"flat"} onClick={() => onChange(page + 1)}>{page + 1}</button>
					</li>
				}
				{ // Only show when the page isn't the last page.
					page + 2 < count &&
					<li>
						<button className={"flat"} onClick={() => onChange(page + 2)}>{page + 2}</button>
					</li>
				}
				{ // Only show when the page isn't the last page.
					page + 3 < count &&
					<li>
						<button className={"flat"} onClick={() => onChange(page + 3)}>{page + 3}</button>
					</li>
				}

				{ // Space between shown pages and last page.
					// Only show when the page isn't the last page.
					page + 4 < count &&
					<li>
						<button className={"flat"} onClick={page + 4 < (count - 1) ? undefined : () => onChange(page + 4)}>{page + 4 < (count - 1) ? "···" : page + 4}</button>
					</li>
				}

				{ // Last page, only show when the current page isn't the last page.
					page < count &&
					<li>
						<button className={"flat"} onClick={() => onChange(count)}>{count}</button>
					</li>
				}
			</ul>

			<Tooltip content={"Next"}>
				<button className={"flat icon-only"} disabled={page >= count} onClick={() => onChange(page + 1)}>
					<CaretRight/>
				</button>
			</Tooltip>
		</nav>
	);
}
